// @flow
import type { 
  VUE_INSTANCE, 
  ROUTER_OBJECT
} from './type'

// 配置vue router 实例
export default function (vueInstance: VUE_INSTANCE): ROUTER_OBJECT {
  const TYPE = process.env.COMPILE_ENV
  let router = {}
  let pushHandler = {}
  let popHandler = {}
  if (TYPE === 'vue') {
    if (!vueInstance.$route || !vueInstance.$router) throw new Error('vue路由api依赖 vue-router')
    pushHandler = {
      get: _ => Object.assign(vueInstance.$router, vueInstance.$route)  // 将vue router中的router route对象 混合
    }
    popHandler = {
      get: _ => _ => history.back()
    }
  } else if (TYPE === 'weex') {
    var navigator = weex.requireModule('navigator')
    if (!navigator) throw new Error('weex路由api依赖 weex navigator')
    pushHandler = {
      get: _ => path => {
        var navigator = weex.requireModule('navigator')
        if (!navigator) throw new Error('weex路由api依赖 weex navigator')
        const url = weex.config.bundleUrll.split('/').slice(0,-1).join('/') + path + '.js';  //獲取b.js | vue編譯後的b.js的相對路徑
        navigator.push({ url, animated: "true" })
      }
    }
    popHandler = {
      get: _ => _ => navigator.pop({ animated: 'true' })
    }
  }
  // 添加push对象
  Reflect.defineProperty(router, 'push', pushHandler)
  // 添加pop对象
  Reflect.defineProperty(router, 'pop', popHandler)
  return router
}